@@ -1,4 +1,4 @@
-/* Copyright © 2007, 2008, 2009, 2010 Jakub Wilk
+/* Copyright © 2007-2015 Jakub Wilk
  * Copyright © 2009 Mateusz Turcza
  *
  * This package is free software; you can redistribute it and/or modify
@@ -1143,6 +1143,30 @@ Cwd::~Cwd()
 
 #endif
 
+#if WIN32
+
+/* class ProgramDir
+ * ================
+ */
+
+ProgramDir::ProgramDir()
+{
+  char buffer[PATH_MAX];
+  size_t n = GetModuleFileName(NULL, buffer, sizeof buffer);
+  if (n == 0)
+    throw_win32_error("GetModuleFileName");
+  if (n >= sizeof buffer)
+  {
+    errno = ENAMETOOLONG;
+    throw_posix_error("GetModuleFileName");
+  }
+  std::string dirname, basename;
+  split_path(buffer, dirname, basename);
+  (*this) += dirname;
+}
+
+#endif
+
 /* utility functions
  * =================
  */
@@ -1289,7 +1313,7 @@ void copy_stream(std::istream &istream, std::ostream &ostream, bool seek, std::s
   }
 }
 
-bool is_stream_a_tty(const std::ostream &ostream)
+bool isatty(const std::ostream &ostream)
 {
   if (&ostream == &std::cout)
     return isatty(STDOUT_FILENO);
@@ -1299,10 +1323,27 @@ bool is_stream_a_tty(const std::ostream &ostream)
      * See http://www.ginac.de/~kreckel/fileno/ for a more general
      * (although unportable, GCC-specific) solution.
      */
-    throw std::invalid_argument("is_a_tty(const std::ostream &)");
+    throw std::invalid_argument("isatty(const std::ostream &)");
   }
 }
 
+void binmode(const std::ostream &ostream)
+{
+#if WIN32
+  if (&ostream == &std::cout)
+  {
+    int rc = setmode(STDOUT_FILENO, O_BINARY);
+    if (rc == -1)
+      throw_posix_error("setmode");
+  }
+  else
+  {
+    /* not implemented */
+    throw std::invalid_argument("binmode");
+  }
+#endif
+}
+
 void split_path(const std::string &path, std::string &directory_name, std::string &file_name)
 {
 #ifdef __MINGW32__
@@ -1312,31 +1353,58 @@ void split_path(const std::string &path, std::string &directory_name, std::strin
    * sophisticated than MinGW32 one, yet it should be sufficient for our
    * purposes.
    */
-  bool append_dot = true;
+  size_t wlength, alength, length = path.length();
+  Array<wchar_t> wpath(length);
+  Array<char> apath(length);
+  wlength = MultiByteToWideChar(
+    CP_ACP, 0,
+    path.c_str(), length,
+    wpath, length
+  );
+  if (wlength == 0)
+    throw_win32_error("MultiByteToWideChar");
   size_t l = 0, r = 0;
-  if (path.length() >= 2 && path[1] == ':')
+  if (wlength >= 2 && wpath[1] == L':')
     l = r = 2;
-  while (l < path.length() && (path[l] == '/' || path[l] == '\\'))
+  while (l < wlength && (wpath[l] == L'/' || wpath[l] == L'\\'))
   {
-    l++, r++;
-    append_dot = false;
+    l++;
+    r++;
   }
-  for (size_t i = l; i < path.length(); i++)
+  for (size_t i = l; i < wlength; i++)
   {
-    if (path[i] == '/' || path[i] == '\\')
+    if (wpath[i] == L'/' || path[i] == L'\\')
     {
       l = i;
       r = i + 1;
-      append_dot = false;
     }
   }
-  directory_name = path.substr(0, l);
-  file_name = path.substr(r);
-  if (append_dot)
-    directory_name += ".";
+  alength = WideCharToMultiByte(
+    CP_ACP, 0,
+    wpath, l,
+    apath, length * 2,
+    NULL, NULL
+  );
+  if (l > 0)
+  {
+    if (alength == 0)
+      throw_win32_error("WideCharToMultiByte");
+    directory_name = std::string(apath, alength);
+  }
+  else
+    directory_name = ".";
+  alength = WideCharToMultiByte(
+    CP_ACP, 0,
+    wpath + r, wlength - r,
+    apath, length * 2,
+    NULL, NULL
+  );
+  if (alength == 0 && r < wlength)
+    throw_win32_error("WideCharToMultiByte");
+  file_name = std::string(apath, alength);
 #else
   /* POSIX-compliant ``basename()`` and ``dirname()`` would split ``/foo/bar/``
-   * into ``/foo`` and ``bar``, instead of desired ``foo/bar`` and an empty
+   * into ``/foo`` and ``bar``, instead of desired ``/foo/bar`` and an empty
    * string. To deal with this weirdness, a trailing ``!`` character is
    * appended to the split path.
    */
