@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2012 Jakub Wilk
+# Copyright © 2009-2015 Jakub Wilk
 #
 # This package is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,10 +13,6 @@ from common import (
 )
 
 class test(case):
-    '''
-    http://sf.net/tracker/?func=detail&aid=1972089&group_id=32953&atid=406583
-    fixed in [5e3937296514]
-    '''
     def test(self):
         self.pdf2djvu('--guess-dpi').assert_()
         r = self.djvudump()
